@@ -1,4 +1,3 @@
-/* $Id: dbdimp.h,v 1.19 2006/09/07 23:24:27 matt Exp $ */
 
 #ifndef _DBDIMP_H
 #define _DBDIMP_H   1
@@ -56,9 +55,7 @@ struct imp_sth_st {
 #define dbd_db_FETCH_attrib     sqlite_db_FETCH_attrib
 #define dbd_db_STORE_attrib_k   sqlite_db_STORE_attrib_k
 #define dbd_db_FETCH_attrib_k   sqlite_db_FETCH_attrib_k
-#ifndef no_last_insert_id
 #define dbd_db_last_insert_id   sqlite_db_last_insert_id
-#endif
 #define dbd_st_prepare          sqlite_st_prepare
 #define dbd_st_rows             sqlite_st_rows
 #define dbd_st_execute          sqlite_st_execute
@@ -72,7 +69,7 @@ struct imp_sth_st {
 #define dbd_st_STORE_attrib_k   sqlite_st_STORE_attrib_k
 #define dbd_st_FETCH_attrib_k   sqlite_st_FETCH_attrib_k
 #define dbd_bind_ph             sqlite_bind_ph
-#define dbd_st_bind_col			sqlite_bind_col
+#define dbd_st_bind_col         sqlite_bind_col
 
 void sqlite3_db_create_function(SV *dbh, const char *name, int argc, SV *func);
 void sqlite3_db_create_aggregate( SV *dbh, const char *name, int argc, SV *aggr );
@@ -92,7 +89,7 @@ newUTF8SVpv(char *s, STRLEN len) {
   sv = newSVpv(s, len);
   SvUTF8_on(sv);
   return sv;
-}  /* End new UTF8SVpv */
+}
 
 static SV *
 newUTF8SVpvn(char *s, STRLEN len) {
@@ -105,7 +102,7 @@ newUTF8SVpvn(char *s, STRLEN len) {
   return sv;
 }
 
-#else  /* SvUTF8_on not defined */
+#else  /* #ifdef SvUTF8_on */
 
 #define newUTF8SVpv newSVpv
 #define newUTF8SVpvn newSVpvn
@@ -113,6 +110,6 @@ newUTF8SVpvn(char *s, STRLEN len) {
 #define SvUTF8_off(a) (a)
 #define sv_utf8_upgrade(a) (a)
 
-#endif
+#endif /* #ifdef SvUTF8_on */
 
-#endif /* _DBDIMP_H */
+#endif /* #ifndef _DBDIMP_H */
